// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.

package transport

import (
_	"errors"
	"path/filepath"
	"io"
	"os"
	"net/url"
	"strings"
	"fmt"
	"compress/gzip"
)

type Buffer struct {
	uri *url.URL
	path string
	exttype string
	fileext string
	readHandle *os.File
	writeHandle *os.File
	gzip bool
	gzipWriter io.WriteCloser
	gzipReader io.ReadCloser
}

func NewBuffer(u *url.URL) (b *Buffer, err error) {
	b = &Buffer{
		uri: u,
		path: filepath.Join(u.Hostname(), u.RequestURI()),
	}
	b.extension()
	b.DetectGzip()

	if b.path == "" || b.path == "-" {
		b.readHandle = os.Stdin
		b.writeHandle = os.Stdout
	} else {
		if b.readHandle, err = os.OpenFile(b.Path(), os.O_RDWR|os.O_CREATE, 0644); err != nil {
			return
		}
		b.writeHandle = b.readHandle
	}

	if b.Gzip() {
		b.gzipWriter = gzip.NewWriter(b.writeHandle)
		if b.gzipReader, err = gzip.NewReader(b.readHandle); err != nil {
			return
		}
	}
	return
}

func (b *Buffer) extension() {
	elements := strings.Split(b.path, ".")
	numberOfElements := len(elements)
	if numberOfElements > 2 {
		b.exttype = elements[numberOfElements - 2]
		b.fileext = elements[numberOfElements - 1]
	}
	b.exttype = elements[numberOfElements - 1]
}

func (b *Buffer) DetectGzip() {
	b.gzip = (b.uri.Query().Get("gzip") == "true" || b.fileext == "gz") 
}

func (b *Buffer) URI() *url.URL {
	return b.uri
}

func (b *Buffer) Path() string {
	return b.path
}

func (b *Buffer) Signature() (documentSignature string) {
	if signatureResp, signatureErr := os.Open(fmt.Sprintf("%s.sig", b.uri.String())); signatureErr == nil {
		defer signatureResp.Close()
		readSignatureBody, readSignatureErr := io.ReadAll(signatureResp)
		if readSignatureErr == nil {
			documentSignature = string(readSignatureBody)
		} else {
			panic(readSignatureErr)
		}
	} else {
		panic(signatureErr)
	}
	return documentSignature
}

func (b *Buffer) ContentType() string {
	return b.exttype
}

func (b *Buffer) SetGzip(gzip bool) {
	b.gzip = gzip
}

func (b *Buffer) Gzip() bool {
	return b.gzip
}

func (b *Buffer) Reader() io.ReadCloser {
	if b.Gzip() {
		return b.gzipReader
	}
	return b.readHandle
}

func (b *Buffer) Writer() io.WriteCloser {
	if b.Gzip() {
		return b.gzipWriter
	}
	return b.writeHandle
}
